var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Tutorial 2 - Adding NumPy, an external component</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Tutorial 2 - Adding NumPy, an external component</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"tutorial-2---adding-numpy-an-external-component\"></a></div>\n\
<p>In this tutorial we add the <a href=\'https://numpy.org/\' title=\'\' target=\'_blank\'>NumPy</a> module to the pyramid project created in <a href=\'#!/url=./developer/tutorials/creating_the_dialog.html\' title=\'\'>Tutorial 1</a>. This tutorial requires that the previous one is completed.</p>\n\
<p>The code of the completed tutorial can be found <a href=\'adding_numpy.html#full-code\' title=\'\'>at the end of this topic</a> for reference.</p>\n\
<div class=\'section\'><a id=\"installing-numpy\"></a><h2 id=\"installing-numpy\">Installing NumPy</h2></div>\n\
<p>We will, in another tutorial, package our pyramid component as a pip package and this will allow us to declare its dependencies and automatically install them.</p>\n\
<p>But at this point the pyramid project is just a bunch of files in a directory and we need to install its dependencies manually.</p>\n\
<p>First, install numpy by running: <span class=\'code\'>pip install numpy</span> in your max Python37 installation</p>\n\
<p><em>(full details <a href=\'#!/url=./developer/python_extension_libraries.html\' title=\'\'>here</a>.)</em></p>\n\
<div class=\'section\'><a id=\"a-note-on-pip\"></a><h2 id=\"a-note-on-pip\">A Note on Pip</h2></div>\n\
<p>The installation of pip for 3ds Max is explained <a href=\'#!/url=./developer/python_extension_libraries.html\' title=\'\'>here</a>.</p>\n\
<p>When pip is installed with --user, it ends up in <span class=\'code\'>/Users/myusername/AppData/Roaming/Python/Python37/Scripts/pip</span> so when we say that after installing pip we want to make sure we use this version by using pip in a path that looks like this:</p>\n\
<p><span class=\'code\'>/Users/myusername/AppData/Roaming/Python/Python37/Scripts/pip install --user something</span></p>\n\
<p>If we don&#39;t use the complete path, there can be other pip installations on the system and <span class=\'code\'>pip install</span> could invoke one of these and either fail or end up at a different location.</p>\n\
<p>When packages are installed with --user for the Python37 3ds Max interpreter, they should end up in <span class=\'code\'>/Users/myusername/AppData/Roaming/Python/Python37/site-packages</span> otherwise it will be impossible to import them.</p>\n\
<p>Ultimately what&#39;s important is to use the right pip, the one installed with 3ds Max.</p>\n\
<div class=\'section\'><a id=\"using-numpy-in-our-project\"></a><h2 id=\"using-numpy-in-our-project\">Using NumPy in our Project</h2></div>\n\
<p>We will add this to <em>graphics.py</em>:</p>\n\
<p>Note: what we do here with numpy is not very useful, the goal is to keep the sample very small.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
import numpy as np\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &#39;&#39;&#39;Construct a pyramid from vertices and faces.&#39;&#39;&#39;\n\
    halfside = side / 2.0\n\
\n\
    npa = np.array([\n\
        [0.0, 0.0, side],\n\
        [-halfside, -halfside, 0.0],\n\
        [-halfside, halfside, 0.0],\n\
        [halfside, 0.0, 0.0]])\n\
\n\
    pyr = rt.mesh(\n\
        vertices=list(map(lambda l: rt.point3(*l.tolist()), npa)),\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr\n\
</pre></div><div class=\'section\'><a id=\"load-the-tutorial-if-we-closed-3ds-max-betwen-the-two-tutorials\"></a><h2 id=\"load-the-tutorial-if-we-closed-3ds-max-betwen-the-two-tutorials\">Load the tutorial (If we closed 3ds Max betwen the two tutorials)</h2></div>\n\
<p>Enter these lines on the listener - all lines need to be entered as separate lines followed by <strong>Ctrl+Enter</strong>.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import sys\n\
sys.path += [ &quot;d:\\\\sources\\\\hacking\\\\python\\\\pyramid&quot; ]\n\
import pyramid.ui\n\
</pre></div><p>Or if the code is already imported from the previous tutorial:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">importlib.reload(pyramid.graphics)\n\
importlib.reload(pyramid.ui)\n\
</pre></div><div class=\'section\'><a id=\"run-the-tutorial\"></a><h2 id=\"run-the-tutorial\">Run the tutorial</h2></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">dialog = pyramid.ui.PyMaxDialog()\n\
dialog.show()\n\
</pre></div><p>We have now modified our pyramid sample to use NumPy. In the <a href=\'#!/url=./developer/tutorials/packaging_pyramid.html\' title=\'\'>next tutorial</a> we will show how to package it with pip.</p>\n\
<div class=\'section\'><a id=\"full-code\"></a><h2 id=\"full-code\">Full Code</h2></div>\n\
<p><strong>graphics.py</strong>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">\n\
&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
import numpy as np\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &#39;&#39;&#39;Construct a pyramid from vertices and faces.&#39;&#39;&#39;\n\
    halfside = side / 2.0\n\
\n\
    npa = np.array([\n\
        [0.0, 0.0, side],\n\
        [-halfside, -halfside, 0.0],\n\
        [-halfside, halfside, 0.0],\n\
        [halfside, 0.0, 0.0]])\n\
\n\
    pyr = rt.mesh(\n\
        vertices=list(map(lambda l: rt.point3(*l.tolist()), npa)),\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";