var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Tutorial 6 - Attaching the VSCode debugger to the 3ds Max process</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Tutorial 6 - Attaching the VSCode debugger to the 3ds Max process</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"tutorial-6---attaching-the-vscode-debugger-to-the-3ds-max-process\"></a></div>\n\
<p>It is possible with VSCode to debug C++ and Python at the same time.</p>\n\
<p>Here is an overview of how this works:</p>\n\
<div class=\'section\'><a id=\"adding-the-proper-extensions-if-not-already-installed-in-vs-code\"></a><h2 id=\"adding-the-proper-extensions-if-not-already-installed-in-vs-code\">Adding the proper extensions (if not already installed in VS Code)</h2></div>\n\
<ul>\n\
<li>The Microsoft Python extension needs to be installed.</li>\n\
<li>The Microsoft C++ extension needs to be added to VSCode (this is free but is not needed to only debug Python programs)</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"adding-debug-configurations\"></a><h2 id=\"adding-debug-configurations\">Adding debug configurations</h2></div>\n\
<ul>\n\
<li>The C++ extension supports multiple debug configurations (from <strong>Debug -&gt; Add Configuration...)</strong>. A good option is the &quot;C/C++ Windows Attach&quot; configuration that lets us attach the debugger to a running process such as <span class=\'code\'>3dsmax.exe</span>. It is possible to choose the process at runtime (<span class=\'code\'>&quot;processId&quot;: &quot;${command:pickProcess}&quot;</span>):</li>\n\
</ul>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">        {\n\
            &quot;name&quot;: &quot;(Windows) Attach&quot;,\n\
            &quot;type&quot;: &quot;cppvsdbg&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;processId&quot;: &quot;${command:pickProcess}&quot;\n\
        },\n\
</pre></div><p>The Python configuration for remote attach is also needed, something like:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">        {\n\
            &quot;name&quot;: &quot;Python: Remote Attach&quot;,\n\
            &quot;type&quot;: &quot;python&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;port&quot;: 5678,\n\
            &quot;host&quot;: &quot;localhost&quot;,\n\
            &quot;pathMappings&quot;: [\n\
                {\n\
                    &quot;localRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
                    &quot;remoteRoot&quot;: &quot;${workspaceFolder}&quot;\n\
                }\n\
            ]\n\
        }\n\
</pre></div><div class=\'section\'><a id=\"starting-the-app-and-enabling-python-debugging\"></a><h2 id=\"starting-the-app-and-enabling-python-debugging\">Starting the app and enabling python debugging</h2></div>\n\
<p>For the Python part of the debugger we still need to enable it manually in our python host by typing this on the listener:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">import ptvsd\n\
ptvsd.enable_attach()\n\
</pre></div><p>For <span class=\'code\'>3dsmax.exe</span>, if we want to attach to the process with the debugger, we need to launch the process manually first, by doing something like <span class=\'code\'>3dsmax.exe -python3</span>.</p>\n\
<div class=\'section\'><a id=\"attaching-to-the-running-application\"></a><h2 id=\"attaching-to-the-running-application\">Attaching to the running application</h2></div>\n\
<p>The two configurations are independently started in VSCode: multiple debuggers can run at the same time and they operate pretty much independently (e.g. when a python breakpoint is hit the C++ part will appear as running).</p>\n\
<p>To attach to the process, first choose the &quot;(Windows) Attach (3dsmax)&quot; item in the debug dropdown and then press the green triangle to start it.</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/StartCpp.png\' title=\'Start C++\'></div>\n\
<p>To attach to python, first choose the &quot;Python: Remote Attach (3dsmax)&quot; item in the debug dropdown and then press the green triangle to start it.</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/StartPython.png\' title=\'Start Python\'></div>\n\
<div class=\'section\'><a id=\"debugging\"></a><h2 id=\"debugging\">Debugging</h2></div>\n\
<p>From the moment the configurations are attached to the running application it becomes possible to select them to debug one or the other aspect of the running application:</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/PythonView.png\' title=\'PythonView\'></div>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/CPPView.png\' title=\'CPPView\'></div>\n\
<p>As shown, at the same moment the application may look as stopped on a python breakpoint and running (not stopped) in C++.</p>\n\
<div class=\'section\'><a id=\"usefulness\"></a><h2 id=\"usefulness\">Usefulness</h2></div>\n\
<p>This dual mode debugging can be especially useful to debug C++ code that invokes the python interpreter (or that deals with it in some way). It is a bit more complicated than only debugging python, and the python part is the same.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";