var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Tutorial 1: Creating and Debugging a UI</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Tutorial 1: Creating and Debugging a UI</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"tutorial-1-creating-and-debugging-a-ui\"></a></div>\n\
<p>In this tutorial we will create and debug a small PySide2 tool for 3ds Max called &quot;pyramid&quot;, using Visual Studio Code and the Python module <span class=\'code\'>ptvsd</span> (Python Tools for Visual Studio Debugger). This tool shows a Qt dialog with a button. Pressing the button adds a black pyramid to the current scene.</p>\n\
<p>The code of the completed tutorial can be found <a href=\'creating_the_dialog.html#full-code\' title=\'\'>at the end of this topic</a> for reference.</p>\n\
<div class=\'section\'><a id=\"prerequisites\"></a><h2 id=\"prerequisites\">Prerequisites</h2></div>\n\
<p>This tutorial series requires that:</p>\n\
<ul>\n\
<li>3ds Max 2021 is installed</li>\n\
<li>3ds Max 2021 is started and using the Python 3 interpreter (the default setting)</li>\n\
<li><a href=\'https://code.visualstudio.com\' title=\'\' target=\'_blank\'>VS Code</a> is installed, and the <a href=\'https://code.visualstudio.com/docs/languages/python\' title=\'\' target=\'_blank\'>Python extension</a> is installed</li>\n\
<li><span class=\'code\'>pip</span> is installed for Python 3 (see <a href=\'#!/url=./developer/python_extension_libraries.html\' title=\'\'>Using pip with Python 3</a>)</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"creating-the-project\"></a><h2 id=\"creating-the-project\">Creating the project</h2></div>\n\
<p>This project will be called pyramid. <a href=\'https://www.python.org/dev/peps/pep-0008/\' title=\'\' target=\'_blank\'>PEP-8</a> (the Style Guide for Python) suggests that modules have small lowercase names.</p>\n\
<p>So somewhere in our file system, create a nested set of directories, both with the name &#39;pyramid&#39;:</p>\n\
<div class=\'codeBlock\'><pre>mkdir -p pyramid/pyramid\n\
</pre></div><p>We will eventually use this directory structure to add proper pip packaging information to our project. For this tutorial we will just create our program in it.</p>\n\
<div class=\'section\'><a id=\"starting-vs-code\"></a><h2 id=\"starting-vs-code\">Starting VS Code</h2></div>\n\
<ul>\n\
<li>Start vs code</li>\n\
<li>Select our top-level <span class=\'code\'>pyramid</span> folder with <strong>File &gt; Open Folder</strong></li>\n\
</ul>\n\
<div class=\'section\'><a id=\"creating-the-source-files\"></a><h2 id=\"creating-the-source-files\">Creating the source files</h2></div>\n\
<p>Our project is small, but we will separate it in two files to simulate a larger project.</p>\n\
<p>Create a new file named <em>ui.py</em> in the <strong>pyramid</strong> subdirectory, and then add the following code:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&quot;&quot;&quot;\n\
    Provide a PySide2 dialog for the pyramid tool.\n\
&quot;&quot;&quot;\n\
from PySide2.QtWidgets import QWidget, QDialog, QLabel, QVBoxLayout, QPushButton\n\
from pymxs import runtime as rt\n\
from .graphics import make_pyramid_mesh\n\
\n\
class PyMaxDialog(QDialog):\n\
    &quot;&quot;&quot;\n\
    Custom dialog attached to the 3ds Max main window\n\
    Message label and action push button to create a pyramid in the 3ds \n\
    Max scene graph\n\
    &quot;&quot;&quot;\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        super(PyMaxDialog, self).__init__(parent)\n\
        self.setWindowTitle(&#39;Pyside2 Qt Window&#39;)\n\
        self.init_ui()\n\
\n\
    def init_ui(self):\n\
        &quot;&quot;&quot; Prepare Qt UI layout for custom dialog &quot;&quot;&quot;\n\
        main_layout = QVBoxLayout()\n\
        label = QLabel(&quot;Click button to create a pyramid in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        btn = QPushButton(&quot;Pyramid&quot;)\n\
        btn.clicked.connect(make_pyramid_mesh)\n\
        main_layout.addWidget(btn)\n\
\n\
        self.setLayout(main_layout)\n\
        self.resize(250, 100)\n\
\n\
</pre></div><ul>\n\
<li>Create another file named <em>graphics.py</em> in the same directory, and add this code:</li>\n\
</ul>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &#39;&#39;&#39;Construct a pyramid from vertices and faces.&#39;&#39;&#39;\n\
    halfside = side / 2.0\n\
    pyr = rt.mesh(\n\
        vertices=[\n\
            rt.point3(0.0, 0.0, side),\n\
            rt.point3(-halfside, -halfside, 0.0),\n\
            rt.point3(-halfside, halfside, 0.0),\n\
            rt.point3(halfside, 0.0, 0.0)\n\
            ],\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr\n\
</pre></div><p>At this point you should have this following directory structure:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-txt\">pyramid/\n\
    pyramid/\n\
        ui.py\n\
        graphics.py\n\
</pre></div><div class=\'section\'><a id=\"wait-i-see-problems-with-my-import-statements\"></a><h2 id=\"wait-i-see-problems-with-my-import-statements\">Wait, I see problems with my import statements</h2></div>\n\
<p>Visual Studio Code uses a linter to verify imports, and by default it does not know where to find the 3ds Max Python modules. The autocomplete system can also have problems understanding our sources.</p>\n\
<p>To solve this issue, point Visual Studio Code to the 3ds Max Python interpreter:</p>\n\
<ul>\n\
<li>Open settings with <strong>File &gt; Preferences &gt; Settings</strong></li>\n\
<li>In the settings page, open <strong>Extensions &gt; Python &gt; Auto Complete: Extra Paths &gt; Edit in settings.json</strong>. This opens the <em>settings.json</em> file.</li>\n\
<li>Add:</li>\n\
</ul>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">    ,\n\
    &quot;python.pythonPath&quot;: &quot;C:\\\\Program Files\\\\Autodesk\\\\3ds Max 2021\\\\Python37\\\\python.exe&quot;,\n\
    &quot;python.autoComplete.extraPaths&quot;: [         \n\
        &quot;C:\\\\Program Files\\\\Autodesk\\\\3ds Max 2021\\\\Python37&quot;     \n\
    ] \n\
\n\
</pre></div><p>More information on this can be found <a href=\'https://code.visualstudio.com/docs/python/environments\' title=\'\' target=\'_blank\'>here</a> and <a href=\'https://code.visualstudio.com/docs/python/editing\' title=\'\' target=\'_blank\'>here</a>.</p>\n\
<div class=\'section\'><a id=\"running-this-first-version-of-the-project-in-3ds-max\"></a><h2 id=\"running-this-first-version-of-the-project-in-3ds-max\">Running this first version of the project in 3ds Max</h2></div>\n\
<p>Switch to 3ds Max, open the listener window and select the Python tab. At that point we should see that Python 3 is used, from the version string displayed in the listener.</p>\n\
<p>By default, the 3ds Max Python interpreter does not know where our sample is, but we want to be able to make it available through <span class=\'code\'>import</span>. So we can type this in the python listener (substituting your own path for the append function):</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import sys\n\
sys.path.append(r&#39;d:\\sources\\hacking\\python\\pyramid&#39;)\n\
</pre></div><p><strong>Note:</strong> Enter each line individually, and use <strong>Ctrl+Enter</strong> to execute them.</p>\n\
<p>Then type:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pyramid.ui\n\
dialog = pyramid.ui.PyMaxDialog()\n\
dialog.show()\n\
</pre></div><p>this should display our dialog: <img src=\'developer/images/tutorials/Dialog.png\' title=\'\'></p>\n\
<p>If you press the <strong>Pyramid</strong> button, you get an execption:</p>\n\
<div class=\'codeBlock\'><pre>&gt;&gt;&gt; Traceback (most recent call last):\n\
  File &quot;d:\\sources\\hacking\\python\\pyramid\\pyramid\\graphics.py&quot;, line 11, in make_pyramid_mesh\n\
    rt.point3(0.0, 0.0, side),\n\
RuntimeError: MAXScript exception raised.\n\
-- Unable to convert: false to type: Float\n\
</pre></div><p>Now we need to debug this problem.</p>\n\
<div class=\'section\'><a id=\"debugging-3ds-max-python-code-from-vs-code\"></a><h2 id=\"debugging-3ds-max-python-code-from-vs-code\">Debugging 3ds Max Python code from VS Code</h2></div>\n\
<p>Before we can use the debugger to debug python code in 3ds Max, we need to install the <a href=\'https://github.com/microsoft/ptvsd\' title=\'\' target=\'_blank\'><span class=\'code\'>ptvsd</span></a> (Python Tools for Visual Studio Debug Server):</p>\n\
<ul>\n\
<li>In your 3ds Max Python 3 interpreter, type: <span class=\'code\'>pip install --user ptvsd</span> &gt; <strong>Note:</strong> <span class=\'code\'>pip</span> should already be installed, see the prerequisites above.</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"enabling-the-debugger-on-the-3ds-max-side\"></a><h3 id=\"enabling-the-debugger-on-the-3ds-max-side\">Enabling the debugger on the 3ds Max side</h3></div>\n\
<p>The ptvsd debug server needs to be running in the 3ds Max Python interpreter. In the 3ds Max Listener window, type:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import ptvsd\n\
ptvsd.enable_attach()\n\
</pre></div><p>this should display:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-txt\">(&#39;0.0.0.0&#39;, 5678)\n\
</pre></div><p>(The previous step will not have to be repeated in our 3ds Max session).</p>\n\
<div class=\'section\'><a id=\"connecting-with-vs-code-the-first-time\"></a><h3 id=\"connecting-with-vs-code-the-first-time\">Connecting with VS Code: the first time</h3></div>\n\
<p>3ds Max is now ready to accept a connection from VS Code, but we must still tell VS Code how to connect to 3ds Max. The first time we want to connect to 3ds Max we need to add a configuration for remote debugging in VS Code.</p>\n\
<ul>\n\
<li>In VS Code, select <strong>Debug &gt; Add Configuration</strong>.</li>\n\
<li>Then choose <strong>Python</strong></li>\n\
<li>The choose <strong>Remote Attach</strong></li>\n\
<li>Then choose <strong>localhost</strong> (the default)</li>\n\
<li>And then <strong>5678</strong> (the default)</li>\n\
</ul>\n\
<p>This will add a configuration code that looks like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">        {\n\
            &quot;name&quot;: &quot;Python: Remote Attach&quot;,\n\
            &quot;type&quot;: &quot;python&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;port&quot;: 5678,\n\
            &quot;host&quot;: &quot;localhost&quot;,\n\
            &quot;pathMappings&quot;: [\n\
                {\n\
                    &quot;localRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
                    &quot;remoteRoot&quot;: &quot;.&quot;\n\
                }\n\
            ]\n\
        },\n\
</pre></div><p>We don&#39;t need to modify this other than we will want to point the <span class=\'code\'>remoteRoot</span> setting directly at our sources</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">                    &quot;remoteRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
</pre></div><p>This configuration only needs to be set up once.</p>\n\
<p>If we want to debug third party modules as well (for example, put breakpoints in pyside2 code), this is not something we will do in this sample but still a very common debugging task, we will also have to enable it in this json file by adding:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">&quot;justMyCode&quot;: false\n\
</pre></div><p>In the end your configuration should look like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-json\">        {\n\
            &quot;name&quot;: &quot;Python: Remote Attach&quot;,\n\
            &quot;type&quot;: &quot;python&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;justMyCode&quot;: false,\n\
            &quot;port&quot;: 5678,\n\
            &quot;host&quot;: &quot;localhost&quot;,\n\
            &quot;pathMappings&quot;: [\n\
                {\n\
                    &quot;localRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
                    &quot;remoteRoot&quot;: &quot;${workspaceFolder}&quot;\n\
                }\n\
            ]\n\
        },\n\
</pre></div><div class=\'section\'><a id=\"connecting-with-vs-code\"></a><h3 id=\"connecting-with-vs-code\">Connecting with VS Code</h3></div>\n\
<p>We need to select our Debug Config in VS Code:</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/DebugConfig.png\' title=\'Debug Config\'></div>\n\
<p>Ensure that our <span class=\'code\'>Python: Remote Attach</span> configuration is selected. When this is the case we can connect the debugger to 3ds Max Python by selecting <strong>Debug &gt; Start Debugging</strong> from the VS Code menu.</p>\n\
<p>If we go back to 3ds Max, we see that the UI is no longer frozen. However, our program is still not running.</p>\n\
<div class=\'section\'><a id=\"running-the-program\"></a><h3 id=\"running-the-program\">Running the program</h3></div>\n\
<p>To run our script we need to type, in the 3ds Max listener window:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">dialog.show()\n\
</pre></div><p>This will run our dialog that does not work the way we want.</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/Dialog.png\' title=\'Dialog\'></div>\n\
<div class=\'section\'><a id=\"adding-a-breakpoint\"></a><h3 id=\"adding-a-breakpoint\">Adding a breakpoint</h3></div>\n\
<p>We know, from a previous run of our program that there is a problem on line 11. So in VS Code we can add a breakpoint on this line by clicking the margin at the left of the line:</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/Breakpoint.png\' title=\'Breakpoint\'></div>\n\
<blockquote>\n\
<p><strong>Note</strong>: if this does not work, it could be that your <span class=\'code\'>remoteRoot</span> path mapping is wrong, or that something else in the previous steps did not go well (for example, your debugger is not running).</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"triggering-the-breakpoint\"></a><h3 id=\"triggering-the-breakpoint\">Triggering the breakpoint</h3></div>\n\
<p>At this point when you click on the <span class=\'code\'>Pyramid</span> button of your dialog, your breakpoint will be reached and you will see it highlighted in VS Code.</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/BreakpointReached.png\' title=\'BreakpointReached\'></div>\n\
<p>You will be able to inspect the locals and you will see that the <span class=\'code\'>side</span> variable is <span class=\'code\'>False</span>, which is not what we want.</p>\n\
<blockquote>\n\
<p><strong>Note</strong>: It is important that you use a dialog that was started during the debugging session. If you try to click the <span class=\'code\'>Pyramid</span> button on a dialog that was started before that, it may not work as expected.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"fixing-the-code\"></a><h3 id=\"fixing-the-code\">Fixing the code</h3></div>\n\
<p>What has happened is that <span class=\'code\'>side</span> is <span class=\'code\'>False</span>. Our code that connects <span class=\'code\'>make_pyramid_mesh()</span> to the dialog button looks like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">        btn.clicked.connect(make_pyramid_mesh)\n\
</pre></div><p>This is wrong because <span class=\'code\'>btn.clicked</span> passes us a Bool that tells us whether the button is checked. Our code receives this bool instead of the default <span class=\'code\'>20.0</span> value that we expected.</p>\n\
<p>So we can edit this code to pass a lambda function that sets <span class=\'code\'>side</span> to 20.0 instead:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">        btn.clicked.connect(lambda: make_pyramid_mesh(20.0))\n\
</pre></div><p>And then we save</p>\n\
<div class=\'section\'><a id=\"making-the-new-code-run\"></a><h3 id=\"making-the-new-code-run\">Making the new code run</h3></div>\n\
<p>To give the control back to max we press <span class=\'code\'>F5</span> (Debug -&gt; Continue). We see that 3ds Max becomes responsive again.</p>\n\
<ul>\n\
<li>Close the dialog</li>\n\
<li>At the listener window&#39;s python command prompt type</li>\n\
<li><div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">import importlib</pre></div></li>\n\
</ul>\n\
<p>and then</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">importlib.reload(pyramid.ui)\n\
</pre></div><p>This displays something like:</p>\n\
<div class=\'codeBlock\'><pre>&lt;module &#39;pyramid.ui&#39; from &#39;d:\\\\sources\\\\hacking\\\\python\\\\pyramid\\\\pyramid\\\\ui.py&#39;&gt;\n\
</pre></div><ul>\n\
<li><p>we must now recreate a brand new version of the dialog by doing:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">dialog = pyramid.ui.PyMaxDialog()</pre></div></li>\n\
<li><p>and then:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-python\">dialog.show()</pre></div></li>\n\
</ul>\n\
<p>Our breakpoint is reached, but this time <span class=\'code\'>side = 20</span> and <span class=\'code\'>halfside = 10</span> and if we hit <span class=\'code\'>F5</span> the code will now successfully complete and show our black pyramid in the viewport.</p>\n\
<p><img src=\'developer/images/tutorials/Result.png\' title=\'\'> ****</p>\n\
<div class=\'section\'><a id=\"full-code\"></a><h2 id=\"full-code\">Full Code</h2></div>\n\
<p><strong>ui.py</strong>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&quot;&quot;&quot;\n\
    Provide a PySide2 dialog for the pyramid tool.\n\
&quot;&quot;&quot;\n\
from PySide2.QtWidgets import QWidget, QDialog, QLabel, QVBoxLayout, QPushButton\n\
from pymxs import runtime as rt\n\
from .graphics import make_pyramid_mesh\n\
\n\
class PyMaxDialog(QDialog):\n\
    &quot;&quot;&quot;\n\
    Custom dialog attached to the 3ds Max main window\n\
    Message label and action push button to create a pyramid in the 3ds \n\
    Max scene graph\n\
    &quot;&quot;&quot;\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        super(PyMaxDialog, self).__init__(parent)\n\
        self.setWindowTitle(&#39;Pyside2 Qt Window&#39;)\n\
        self.init_ui()\n\
\n\
    def init_ui(self):\n\
        &quot;&quot;&quot; Prepare Qt UI layout for custom dialog &quot;&quot;&quot;\n\
        main_layout = QVBoxLayout()\n\
        label = QLabel(&quot;Click button to create a pyramid in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        btn = QPushButton(&quot;Pyramid&quot;)\n\
        btn.clicked.connect(lambda: make_pyramid_mesh(20.0))\n\
        main_layout.addWidget(btn)\n\
\n\
        self.setLayout(main_layout)\n\
        self.resize(250, 100)\n\
</pre></div><p><strong>graphics.py</strong>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &#39;&#39;&#39;Construct a pyramid from vertices and faces.&#39;&#39;&#39;\n\
    halfside = side / 2.0\n\
    pyr = rt.mesh(\n\
        vertices=[\n\
            rt.point3(0.0, 0.0, side),\n\
            rt.point3(-halfside, -halfside, 0.0),\n\
            rt.point3(-halfside, halfside, 0.0),\n\
            rt.point3(halfside, 0.0, 0.0)\n\
            ],\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";