var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Tutorial 3 - Packaging our pyramid module as a pip package</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Tutorial 3 - Packaging our pyramid module as a pip package</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"tutorial-3---packaging-our-pyramid-module-as-a-pip-package\"></a></div>\n\
<p>In this tutorial we create a pip package from the code of the pyramid tutorial.</p>\n\
<p>The information provided here is similar to the information provided in the official <a href=\'https://packaging.python.org/tutorials/packaging-projects\' title=\'\' target=\'_blank\'>Packaging Python Projects documentation</a>.</p>\n\
<p>Before we start be sure you have completed the steps of <a href=\'#!/url=./developer/tutorials/creating_the_dialog.html\' title=\'\'>the first tutorial</a> and <a href=\'#!/url=./developer/tutorials/adding_numpy.html\' title=\'\'>the second tutorial</a>.</p>\n\
<div class=\'section\'><a id=\"create-an-init-file\"></a><h2 id=\"create-an-init-file\">Create an <strong>init</strong> file</h2></div>\n\
<p>First, create an <span class=\'code\'>__init__.py</span> file in <em>pyramid/pyramid</em>.</p>\n\
<p>This file should contain:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">name = &quot;pyramid&quot;\n\
</pre></div><div class=\'section\'><a id=\"create-license-and-readme.md-files\"></a><h2 id=\"create-license-and-readme.md-files\">Create LICENSE and README.md files</h2></div>\n\
<p>Our license file can contain something like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-txt\">Copyright (c) 2019 Autodesk, all rights reserved.\n\
</pre></div><p>And our README.md can contain something like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-md\"># Pyramid\n\
End to end example for constructing a python component for 3ds Max.\n\
\n\
- uses PySide2 &amp; NumPy\n\
- adds a small dialog to 3ds Max\n\
</pre></div><div class=\'section\'><a id=\"create-a-setup.py-file\"></a><h2 id=\"create-a-setup.py-file\">Create a <span class=\'code\'>setup.py</span> file</h2></div>\n\
<p>Create a new setup.py file with the following contents:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import setuptools\n\
\n\
with open(&quot;README.md&quot;, &quot;r&quot;) as fh:\n\
    long_description = fh.read()\n\
\n\
setuptools.setup(\n\
    name=&quot;pyramid-autodesk&quot;,\n\
    version=&quot;0.0.1&quot;,\n\
    author=&quot;Autodesk Dude&quot;,\n\
    author_email=&quot;some.dude@autodesk.com&quot;,\n\
    description=&quot;A sample 3ds Max Python Package&quot;,\n\
    long_description=long_description,\n\
    long_description_content_type=&quot;text/markdown&quot;,\n\
    url=&quot;https://git.autodesk.com/windish/maxpythontutorials&quot;,\n\
    packages=setuptools.find_packages(),\n\
    classifiers=[\n\
        &quot;Programming Language :: Python :: 3&quot;,\n\
        &quot;LICENSE :: OTHER/PROPRIETARY LICENSE&quot;,\n\
        &quot;Operating System :: Microsoft :: Windows&quot;\n\
    ],\n\
    python_requires=&#39;&gt;=3.7&#39;\n\
)\n\
</pre></div><div class=\'section\'><a id=\"installing-from-a-file-location-for-development-editable-install\"></a><h2 id=\"installing-from-a-file-location-for-development-editable-install\">Installing from a file location for development (editable install)</h2></div>\n\
<p>We may or may not want to make our 3ds Max extension public. For this example we will keep it private. In this scenario we will not upload our extension to the PyPI repository, but we can still use the convenience of pip to install our package.</p>\n\
<p>Assuming that pyramid is in a subdirectory of the current working directory, we can run:</p>\n\
<div class=\'codeBlock\'><pre>pip install --user -e ./pyramid\n\
</pre></div><p>This will install pyramid in such a way that it can be still edited locally but imported (and reloaded) without having to edit the system path (as we did in tutorial 1).</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import sys\n\
sys.path.append(&quot;d:\\\\sources\\\\hacking\\\\python\\\\pyramid&quot;)\n\
</pre></div><p>We can now make an edit to our project and use <span class=\'code\'>importlib.reload()</span> to reload it.</p>\n\
<p>So for example, we could change a line in <em>ui.py</em> from:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">        self.setWindowTitle(&#39;Pyside2 Qt Window&#39;)\n\
</pre></div><p>to:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">        self.setWindowTitle(&#39;Pyramid&#39;)\n\
</pre></div><p>And then, in the 3ds Max Listener:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">importlib.reload(pyramid.ui)\n\
# or alternately\n\
# import(pyramid.ui)\n\
</pre></div><p>And then:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">dialog = pyramid.ui.PyMaxDialog()\n\
dialog.show()\n\
</pre></div><p>We should now see our renamed version:</p>\n\
<div class=\'figure\'><img src=\'developer/images/tutorials/Renamed.png\' title=\'Renamed\'></div>\n\
<div class=\'section\'><a id=\"installing-from-a-file-location-not-editable\"></a><h2 id=\"installing-from-a-file-location-not-editable\">Installing from a file location (not editable)</h2></div>\n\
<p>Assuming that pyramid is in a subdirectory of the current working directory, we can run:</p>\n\
<div class=\'codeBlock\'><pre>pip install --user ./pyramid\n\
</pre></div><p>The package installed this way will not be editable, meaning if we modify the working version, the installed version will not be affected unless it is re-installed.</p>\n\
<div class=\'section\'><a id=\"other-installations\"></a><h2 id=\"other-installations\">Other installations</h2></div>\n\
<p>Pip is normally intended for network base installation from PyPI. This scenario is detailed in the <a href=\'http://packaging.python.org/tutorials/packaging-projects\' title=\'\' target=\'_blank\'>official python documentation</a>.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";