var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Using pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Using pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"using-pymxs\"></a></div>\n\
<p>The <span class=\'code\'>pymxs</span> Python module provides a wrapper for the MAXScript engine, and exposes all the interfaces, globals, and classes available in MAXScript. This set of topics covers the basics of how to work with 3ds Max scenes and objects using <span class=\'code\'>pymxs</span>. Because <span class=\'code\'>pymxs</span> is a light wrapper for MAXScript, this guide does not contain a description of every available API, but rather some overview and examples. Virtually every API described in the MAXScript Guide can be accessed with <span class=\'code\'>pymxs</span> (with some exceptions outlined in the <a href=\'#!/url=./developer/using_pymxs/pymxs_differences.html\' title=\'\'>Differences Between pymxs and MAXScript</a> topic).</p>\n\
<p>Note that scripts must import the pymxs module when run in Python 3. In Python 2 this is optional, as pymxs is imported automatically when 3ds Max starts.</p>\n\
<p>3ds Max ships with several example scripts that illustrate how to perform various common tasks using <span class=\'code\'>pymxs</span>. These samples are located in <span class=\'code\'>[3ds Max Install]/scripts/PythonSamples/</span>.</p>\n\
<div class=\'section\'><a id=\"case-sensitivity\"></a><h2 id=\"case-sensitivity\">Case Sensitivity</h2></div>\n\
<p>While Python is case sensitive (so that, for example <span class=\'code\'>print()</span> and <span class=\'code\'>Print()</span> are not equivalent), MAXScript is not. The convention in the MAXScript documentation is to generally camelCase functions and properties to make them easier to read, but this is not always the case in samples, or in scripts you may find elsewhere. Because they are essentially MAXScript calls, members in <span class=\'code\'>pymxs.runtime</span> are <strong>not</strong> case sensitive. Both <span class=\'code\'>pyxms.runtime.converttomesh()</span> and <span class=\'code\'>pymxs.runtime.convertToMesh()</span> are valid, which may seem odd to someone used to Python.</p>\n\
<div class=\'section\'><a id=\"indices\"></a><h2 id=\"indices\">Indices</h2></div>\n\
<p>Another difference between Python and MAXScript is how array indices are managed. Python is 0-based, MAXScript is 1-based. When arrays are passed between Python and MAXScript, they are converted to the native index format. For example, when accessing the <span class=\'code\'>meditMaterials</span> array of material editor slots from <span class=\'code\'>pymxs</span>, use 0-based indices:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; len(pymxs.runtime.meditmaterials)\n\
24\n\
&gt;&gt;&gt; pymxs.runtime.meditmaterials[0]\n\
&lt;Standardmaterial&lt;01 - Default:Standard&gt;&gt;\n\
</pre></div><p>However, when using functions or methods that take index arguments, the indices are 1-based even in <span class=\'code\'>pymxs</span>. For example, the <span class=\'code\'>setMeditMaterial(&lt;slot_index&gt;, &lt;material&gt;)</span> function takes as its first argument the index of a material slot. In this case, to set the first slot:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; mat = pymxs.runtime.PhysicalMaterial()\n\
&gt;&gt;&gt; pymxs.runtime.setMeditMaterial(1, mat)\n\
</pre></div><div class=\'section\'><a id=\"equality-vs-identity\"></a><h2 id=\"equality-vs-identity\">Equality vs Identity</h2></div>\n\
<p>The <span class=\'code\'>is</span> operator is often used in Python to determine equality. However, objects accessed through pymxs are wrapped by the pymxs layer, and though may refer to the same scene object, are not identical. Here is an example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as mxs\n\
\n\
t = mxs.teapot(name=&#39;my teapot&#39;)\n\
t2 = mxs.getNodeByName(&#39;my teapot&#39;)\n\
print (t is t2) # this is false\n\
print (t == t2) # this is true\n\
print (f&#39;The IDs are different: t:{id(t)} t2:{id(t2)}&#39;)\n\
</pre></div><p>As this example shows, the equality of pymxs objects should be tested using the <span class=\'code\'>==</span> operator rather than <span class=\'code\'>is</span>.</p>\n\
<div class=\'section\'><a id=\"translating-maxscript-syntax-to-python\"></a><h2 id=\"translating-maxscript-syntax-to-python\">Translating MAXScript Syntax to Python</h2></div>\n\
<p>While the <span class=\'code\'>pymxs</span> module is a thin wrapper for MAXScript, it is not always obvious how to translate certain MAXScript syntax into the Python world. Here are some of the common cases:</p>\n\
<div class=\'section\'><a id=\"keyword-arguments\"></a><h3 id=\"keyword-arguments\">Keyword Arguments</h3></div>\n\
<p>Many MAXScript functions take keyword parameters. Consider this example that creates a bitmap and renders to it. It uses two keyword parameters, <span class=\'code\'>color:</span> when constructing the Bitmap, and <span class=\'code\'>to:</span> when rendering.</p>\n\
<div class=\'codeBlock\'><pre>bm = Bitmap 320 240 color:white\n\
render to:bm\n\
display bm\n\
</pre></div><p>The equivalent in pymxs also uses keyword arguments, and looks like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">bm = pymxs.runtime.Bitmap(320,240,color=pymxs.runtime.white)\n\
pymxs.runtime.render(to=bm)\n\
pymxs.runtime.display(bm)\n\
</pre></div><div class=\'section\'><a id=\"maxscript-names\"></a><h3 id=\"maxscript-names\">MAXScript Names</h3></div>\n\
<p>Name literals are commonly used in MAXScript to signify options passed to functions, and have no corresponding type in Python. They can be created with <span class=\'code\'>pymxs.runtime.Name()</span>. For example, to use the equivalent of <span class=\'code\'>toolmode.coordsys #world</span>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.toolMode.coordsys(pymxs.runtime.Name(&quot;world&quot;))\n\
</pre></div><p>Many functions return names or arrays of names. For example, to get all the properties for a box, using the equivalent of <span class=\'code\'>getpropnames box</span>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">boxprops = pymxs.runtime.getpropnames(pymxs.runtime.box)\n\
</pre></div><p>This yields a MAXScript Array object containing runtime.Name objects. In this case, it might be more useful to generate a native list of Python strings:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">boxprops = [str(x) for x in list(pymxs.runtime.getpropnames(pymxs.runtime.box))]\n\
# gives: [&#39;typeinCreationMethod&#39;, &#39;typeInPos&#39;, &#39;typeInLength&#39;, &#39;typeInWidth&#39;, &#39;typeInHeight&#39;, &#39;length&#39;, &#39;width&#39;, &#39;height&#39;, &#39;widthsegs&#39;, &#39;lengthsegs&#39;, &#39;heightsegs&#39;, &#39;mapcoords&#39;]\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";