var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>By-Reference Parameter Handling in pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>By-Reference Parameter Handling in pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"by-reference-parameter-handling-in-pymxs\"></a></div>\n\
<p>Values passed back to Python from pymxs MAXScript functions that take a by-reference parameter can be translated to Python values using the <span class=\'code\'>pymxs.byref()</span> and <span class=\'code\'>pymxs.mxsreference()</span> functions. In MAXScript, &#39;by reference&#39; means that a parameter is defined and has an initial value before it is passed to a function (known as an &#39;in value&#39;), and may be modified by the function operation (known as an &#39;out value&#39;). By reference parameters are indicated with a &#39;&amp;&#39; character. For more information about this feature, see the &quot;By Reference Parameter Passing&quot; topic in the MAXScript Help. This behavior has no direct parallel in Python.</p>\n\
<p>The <span class=\'code\'>pymxs.byref()</span> function is introduced in <strong>3ds Max 2021</strong>, and is the preferred way to work with by-reference parameters, as it provides a more &#39;Pythonic&#39; way of handling these parameters.</p>\n\
<div class=\'section\'><a id=\"pymxs.byref\"></a><h2 id=\"pymxs.byref\">pymxs.byref()</h2></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.byref(PythonValue | MXSValue)\n\
</pre></div><p>This function marks a value to be used as a by-reference parameter. When you mark a parameter with <span class=\'code\'>byref()</span>, the called <span class=\'code\'>pymxs.runtime</span> function returns a tuple that contains, in order, the result of the function call, and then the updated value of each <span class=\'code\'>byref()</span> parameter. Values passed to <span class=\'code\'>byref()</span> are not modified in place, as they would be in the equivalent MAXScript function call. In other words, parameters marked with <span class=\'code\'>byref()</span> are not updated by the function call, even if they are considered &quot;output parameters&quot;. The updated value is contained in the returned tuple.</p>\n\
<p>If a by-reference parameter is only an &quot;output parameter&quot;, an actual value does not need to be supplied to <span class=\'code\'>byref()</span>, the value can be passed as <span class=\'code\'>None</span>. If the parameter is an &quot;input parameter&quot; or &quot;input and output parameter&quot;, a value does need to be supplied.</p>\n\
<p>The values passed to <span class=\'code\'>byref()</span> can be either Python values or MAXScript values.</p>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
b = rt.box()\n\
# call cloneNodes(), which takes a named by-reference parameter &#39;newNodes&#39;\n\
# this is only an out parameter, so we can pass it as None\n\
result, myClonedNodes = rt.maxOps.cloneNodes(b,  cloneType=rt.Name(&#39;reference&#39;), newNodes=pymxs.byref(None))\n\
print(&#39;Result: {}, clonedNodes: {}&#39;.format(result, myClonedNodes))\n\
\n\
# colorById also takes an out parameter, but it is not named:\n\
res, my_color = rt.maxOps.colorById(2, pymxs.byref(None))\n\
print(&#39;my_color: &#39;, my_color)\n\
\n\
# when dealing with an in AND out parameter, the value is returned in the tuple\n\
# NOT modified in place, as it is done in MAXScript\n\
theFileName = &#39;Simple_Stone_Mtl_Marble_bump.jpg&#39;\n\
result, theFilePath = rt.FileResolutionManager.getFullFilePath(pymxs.byref(theFileName), rt.name(&#39;bitmap&#39;))\n\
print (&#39;the input parameter was not changed: &#39;, theFileName)\n\
print (&#39;the returned output parameter is set to the full path: &#39;,  theFilePath)\n\
</pre></div><p>The output is:</p>\n\
<div class=\'codeBlock\'><pre>Result: True, clonedNodes: [&lt;Box&lt;$Box:Box004 @ [0.000000,0.000000,0.000000]&gt;&gt;]\n\
my_color:  (color 255 0 0)\n\
the input parameter was not changed:  Simple_Stone_Mtl_Marble_bump.jpg\n\
the returned output parameter is set to the full path:  C:\\Program Files\\3ds Max 2021\\Maps\\adskMtl\\Simple_Stone_Mtl_Marble_bump.jpg\n\
</pre></div><div class=\'section\'><a id=\"pymxs.mxsreference-deprecated\"></a><h3 id=\"pymxs.mxsreference-deprecated\">pymxs.mxsreference() <em>Deprecated</em></h3></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.mxsreference(PythonList | MXSValue)\n\
<blockquote>\n\
</blockquote></pre></div><p><strong>Note:</strong> As of <strong>3ds Max 2021</strong>, this function is marked as deprecated, and will produce a warning in the Listener window when called the first time. Use <span class=\'code\'>pymxs.byref()</span> instead.</p>\n\
\n\
<p>If the parameter is an &quot;out parameter&quot; (meaning the function updates its value), it will be updated if:</p>\n\
<ul>\n\
<li>it is a Python list</li>\n\
<li>it is a MXSValue that is not auto-converted to a Python value</li>\n\
</ul>\n\
<p>It will not be updated if:</p>\n\
<ul>\n\
<li>it is a Python value (other than list)</li>\n\
<li>it is a MXSValue that is auto-converted to a Python value</li>\n\
</ul>\n\
<p>The following MAXScript types are auto-converted to Python types: string, int, int64, intptr, float, double, boolean, objectset, collection (but not bitarray or MXSModifierArray).</p>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
from pymxs import runtime as rt\n\
\n\
my_sphere = rt.sphere(radius=10)\n\
my_type = rt.name(&quot;reference&quot;)\n\
\n\
offset1 = rt.Point3(50.0, 50.0, 10.0)\n\
# create a Python list to update with the new nodes:\n\
my_returned_list = []\n\
# pass it using mxsference() so it can be updated:\n\
rt.maxOps.cloneNodes(my_sphere,  offset=offset1, cloneType=my_type, newNodes=pymxs.mxsreference(my_returned_list))\n\
print(&quot;python array: {}&quot;.format(my_returned_list))\n\
\n\
# we can also use a MAXScript Array object to pass:\n\
\n\
rt.mra = rt.array()\n\
my_result_array = rt.mra\n\
offset2 = rt.Point3(-50.0, -50.0, 10.0)\n\
rt.maxOps.cloneNodes(my_sphere,  offset=offset2, cloneType=my_type, newNodes=pymxs.mxsreference(my_result_array))\n\
print(&quot;runtime array: {}&quot;.format(my_result_array))\n\
</pre></div><p>The result printed to the Listener is:</p>\n\
<div class=\'codeBlock\'><pre>python array: \n\
[&lt;Sphere&lt;$Sphere:Sphere005 @ [50.000000,50.000000,10.000000]&gt;&gt;]\n\
runtime array: \n\
#($Sphere:Sphere006 @ [-50.000000,-50.000000,10.000000])\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";