var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Performing Inspection of Classes and Objects in pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Performing Inspection of Classes and Objects in pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"performing-inspection-of-classes-and-objects-in-pymxs\"></a></div>\n\
<p>There are several functions you can use to inspect objects and classes in pymxs. This topic discusses the most commonly used functions, but there are many more. See the MAXScript Help topic &#39;Class and Object Inspector Functions&#39;, and &#39;showInterfaces Inspector Function&#39; for more detailed information.</p>\n\
<div class=\'section\'><a id=\"apropos\"></a><h2 id=\"apropos\"><span class=\'code\'>apropos()</span></h2></div>\n\
<p>This function searches and prints out information about global variables by name pattern and class of contents. Globals include things like classes, structs (such as polyop, which contains functions for working with EditablePoly objects), and functions. The <span class=\'code\'>apropos()</span> function takes three parameters:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.apropos(pattern_string, [implicitWild=true|false], [to=pymxs_stream])\n\
</pre></div><ul>\n\
<li><em>pattern_string</em> is a string used to match the names of globals.</li>\n\
<li><em>implicitWild</em> is optional, and specifies whether there is an implicit wildcard character at the beginning and end of the pattern string. The default is true.</li>\n\
<li><em>to</em> specifies an optional stream to send output to.</li>\n\
</ul>\n\
<blockquote>\n\
<p><strong>Note:</strong> Functions that would send output to the listener when executed in MAXScript do not do so in pymxs. For this reason the optional <em>to</em> parameter is actually required in order to obtain useful information from this function.</p>\n\
</blockquote>\n\
<p>For example, to get all the globals that contain the string &#39;plane&#39;:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">res=pymxs.runtime.stringStream(&#39;&#39;)\n\
pymxs.runtime.apropos(&#39;plane&#39;,to=res)\n\
print(res)\n\
</pre></div><p>Which produces this output:</p>\n\
<div class=\'codeBlock\'><pre>StringStream:&#39;PxUpdateGroundPlane (MAXScriptFunction): PxUpdateGroundPlane()\n\
gw.dualPlane (&lt;internal&gt;): SystemGlobal:dualPlane\n\
meshOps.startSlicePlane (Primitive): startSlicePlane()\n\
Planet (const MAXClass): Planet\n\
meshop.moveVertsToPlane (Primitive): moveVertsToPlane()\n\
PFPlaneMeshWrapper (const MAXClass): PFPlaneMeshWrapper\n\
g_thePlane (UndefinedClass): undefined\n\
Plane_Angle (const MAXClass): Plane_Angle\n\
PlaneAngleManip (const MAXClass): Plane_Angle\n\
polyOps.resetPlane (Primitive): resetPlane()\n\
polyOps.startSlicePlane (Primitive): startSlicePlane()\n\
Plane (const MAXClass): Plane\n\
skinOps.gizmoResetRotationPlane (Primitive): gizmoResetRotationPlane()\n\
SectionPlane (const UndefinedClass): undefined\n\
polyop.moveFacesToPlane (Primitive): moveFacesToPlane()\n\
polyop.resetSlicePlane (Primitive): resetSlicePlane()\n\
polyop.getSlicePlane (Primitive): getSlicePlane()\n\
polyop.setSlicePlane (Primitive): setSlicePlane()\n\
polyop.inSlicePlaneMode (Primitive): inSlicePlaneMode()\n\
polyop.moveEdgesToPlane (Primitive): moveEdgesToPlane()\n\
polyop.moveVertsToPlane (Primitive): moveVertsToPlane()\n\
</pre></div><p>To refine this search to only those globals beginning with &#39;plane&#39;, we can turn off implicit wildcards, and supply one only at the end of the string:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">res=pymxs.runtime.stringStream(&#39;&#39;)\n\
pymxs.runtime.apropos(&#39;plane*&#39;, implicitWild=false, to=res)\n\
print(res)\n\
</pre></div><p>Which produces a much shorter output:</p>\n\
<div class=\'codeBlock\'><pre>Planet (const MAXClass): Planet\n\
Plane_Angle (const MAXClass): Plane_Angle\n\
PlaneAngleManip (const MAXClass): Plane_Angle\n\
Plane (const MAXClass): Plane\n\
</pre></div><div class=\'section\'><a id=\"showclass-and-showproperties\"></a><h2 id=\"showclass-and-showproperties\"><span class=\'code\'>showClass()</span> and <span class=\'code\'>showProperties()</span></h2></div>\n\
<p>The <span class=\'code\'>showClass()</span> and <span class=\'code\'>showProperties()</span> functions are useful for inspecting specific pymxs classes and their properties. The <span class=\'code\'>showClass()</span> function takes a string, while <span class=\'code\'>showProperties()</span> takes an object. The <span class=\'code\'>showProperties()</span> function has the advantage that it can show dynamic properties that might have been created on the object after it was created.</p>\n\
<p>The <span class=\'code\'>showClass()</span> function has the following form:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.showClass(pattern_string, [to=pymxs_stream])\n\
</pre></div><ul>\n\
<li><em>pattern_string</em> is a string to match against class names. By default this is an exact match, so &#39;box&#39; will only match the <span class=\'code\'>Box</span> class. You can add wildcards to expand the match. For example, &#39;box*&#39; will also match <span class=\'code\'>BoxGizmo</span> and <span class=\'code\'>Box_2_Global_Utility</span>.</li>\n\
<li><em>to</em> is an optional output stream, though in the pymxs context it is the only way to get useful output from this function.</li>\n\
</ul>\n\
<p>The placement of wildcards in the pattern string affects the results found:</p>\n\
<ul>\n\
<li>&#39;path*&#39; -- all 3ds Max classes starting with &#39;path&#39;</li>\n\
<li>&#39;noise.*&#39; -- all the accessible properties on the Noise texture map</li>\n\
<li>&#39;<em>:mod</em>&#39; -- all the modifier classes</li>\n\
<li>&#39;<em>.</em>rad*&#39; -- all the classes with a property name containing &#39;rad&#39;</li>\n\
<li>&#39;<em>:</em>controller*&#39; -- all the classes that have &#39;controller&#39; in their superclass name</li>\n\
</ul>\n\
<p>For example, to get the classID and properties of the Sphere class:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">res = pymxs.runtime.stringStream(&#39;&#39;)\n\
pymxs.runtime.showClass(&#39;sphere.*&#39;, to=res)\n\
print(res)\n\
</pre></div><p>Which results in:</p>\n\
<div class=\'codeBlock\'><pre>Sphere : GeometryClass {11,0}\n\
  .typeinCreationMethod (Creation_Method) : integer\n\
  .typeInPos (Type_in_Position) : point3\n\
  .typeInRadius (Radius) : float\n\
  .radius : worldUnits\n\
  .segs (Segments) : integer\n\
  .smooth : boolean\n\
  .hemisphere : float\n\
  .chop (Squash) : integer\n\
  .recenter : boolean\n\
  .mapCoords (Generate_Texture_Coords) : boolean\n\
  .slice (Slice_On/sliceon) : boolean\n\
  .sliceFrom (Slice_From) : angle\n\
  .sliceTo (Slice_To) : angle\n\
</pre></div><p>The <span class=\'code\'>showProperties()</span> function displays the properties of an object that is a specific instance of a class. It has the form:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.showProperties(pymxs_object, [pattern_string], [to=pymxs_stream])\n\
</pre></div><ul>\n\
<li><em>pymxs_object</em> is an object that derives from the MAXWrapper class</li>\n\
<li><em>pattern_string</em> is an optional wildcard pattern to apply to properties to be inspected</li>\n\
<li><em>to</em> is an optional output stream, though in the pymxs context it is the only way to get useful output from this function.</li>\n\
</ul>\n\
<p>For example, to get all the properties for a sphere object:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">s = pymxs.runtime.sphere()\n\
res = pymxs.runtime.stringStream(&#39;&#39;)\n\
pymxs.runtime.showProperties(s, to=res)\n\
print(res)\n\
</pre></div><p>After which the result stream <span class=\'code\'>res</span> contains:</p>\n\
<div class=\'codeBlock\'><pre>  .typeinCreationMethod (Creation_Method) : integer\n\
  .typeInPos (Type_in_Position) : point3\n\
  .typeInRadius (Radius) : float\n\
  .radius : worldUnits\n\
  .segs (Segments) : integer\n\
  .smooth : boolean\n\
  .hemisphere : float\n\
  .chop (Squash) : integer\n\
  .recenter : boolean\n\
  .mapCoords (Generate_Texture_Coords) : boolean\n\
  .slice (Slice_On/sliceon) : boolean\n\
  .sliceFrom (Slice_From) : angle\n\
  .sliceTo (Slice_To) : angle\n\
  .realWorldMapSize : boolean\n\
</pre></div><div class=\'section\'><a id=\"showinterfaces\"></a><h2 id=\"showinterfaces\"><span class=\'code\'>showInterfaces()</span></h2></div>\n\
<p>In pymxs, the methods that an object can use are contained in one or more interfaces associated with that object&#39;s class. The <span class=\'code\'>showInterfaces()</span> function can be used to inspect those interfaces and the methods they contain. The <span class=\'code\'>showIntefaces()</span> function has the form:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">showInterfaces(pymxs_object, [to=pymxs_stream])\n\
</pre></div><ul>\n\
<li><em>pymxs_object</em> is the 3ds Max object, class or node to be inspected</li>\n\
<li><em>to</em> is an optional output stream, though in the pymxs context it is the only way to get useful output from this function.</li>\n\
</ul>\n\
<p>For example to view all the interfaces and associated methods on an Editable_Poly object:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">t = pymxs.runtime.teapot()\n\
pymxs.runtime.convertTo(t, pymxs.runtime.editable_Poly)\n\
res = pymxs.runtime.stringStream(&#39;&#39;)\n\
pymxs.runtime.showInterfaces(t, to=res)\n\
print(res)\n\
</pre></div><p>The <span class=\'code\'>res</span> stringStream will contain a large number of methods, as the EditablePoly interface exposes a lot of functionality:</p>\n\
<div class=\'codeBlock\'><pre> Interface: EditablePoly\n\
   Properties:\n\
    .edgeChamferType : enum : Read|Write\n\
       edgeChamferType enums: {#Standard|#Quad}\n\
    .edgeChamferQuadIntersections : boolean : Read|Write\n\
   Methods:\n\
    &lt;bool&gt;Hide &lt;enum&gt;mnSelLevel flags:&lt;DWORD&gt;\n\
       mnSelLevel enums: {#Object|#Vertex|#Edge|#Face|#CurrentLevel}\n\
       flags default value: 1\n\
    &lt;bool&gt;unhideAll &lt;enum&gt;mnSelLevel\n\
       mnSelLevel enums: {#Object|#Vertex|#Edge|#Face|#CurrentLevel}\n\
    &lt;void&gt;namedSelCopy &lt;string&gt;Name\n\
    \n\
    ... etc\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";