var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Working with Macroscripts and Menus</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Working with Macroscripts and Menus</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"working-with-macroscripts-and-menus\"></a></div>\n\
<p>You can work with 3ds Max menus using these interfaces:</p>\n\
<ul>\n\
<li><span class=\'code\'>menuMan</span></li>\n\
<li><span class=\'code\'>menuItem</span></li>\n\
<li><span class=\'code\'>quadMenu</span></li>\n\
</ul>\n\
<blockquote>\n\
<p><strong>Note:</strong> Be very careful when using or testing the <span class=\'code\'>menuMan</span> API. It makes permanent changes to the menu database, and it is very easy to corrupt the menus. For example, if you &quot;unRegister&quot; the main menu bar, 3ds Max won&#39;t start. Before using this API, make a copy of the &quot;..\\UI\\MaxStartUI.mnu&quot; file so you can easily restore it if any problems arise.</p>\n\
</blockquote>\n\
<p>A menu item is typically associated with a MacroScript, which is executed when that menu item is pressed. In the following example we create a Python function, connect it to a pymxs runtime global variable, and then define a macroscript using <span class=\'code\'>macros.new()</span>. In this case, we simply invoke our Python function.</p>\n\
<p>Next, we find the <strong>Help</strong> menu using menuMan.find(), create a new menuitem that invokes our macroscript, and add it to the Help menu.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
# Our Py function:\n\
def myfunc():\n\
    print(&#39;hello world&#39;)\n\
\n\
# Connect to a gobal in the runtime:\n\
rt.mxs_hello = myfunc\n\
\n\
macroscript_name = &#39;My_Macroscript&#39;\n\
macroscript_category = &#39;Test&#39;\n\
macroscript_tooltip = &#39;This is a tooltip&#39;\n\
# this sets the text used for any menus or ui controls\n\
# associated with this macroscript\n\
macroscript_text = &#39;My Macroscript&#39;\n\
# this is a MAXSCript:\n\
macroscript_content = &#39;mxs_hello()&#39; \n\
\n\
macro_id = rt.macros.new(macroscript_category, macroscript_name, macroscript_tooltip, macroscript_button, macroscript_content)\n\
\n\
# Now we can create a menu item for this script\n\
help_menu = rt.menuMan.findMenu(&#39;&amp;Help&#39;)\n\
menu_item = rt.menuMan.createActionItem(macroscript_name, macroscript_category)\n\
help_menu.addItem(menu_item, -1)\n\
rt.menuMan.updateMenuBar()\n\
\n\
# this can be removed by:\n\
# help_menu.removeItem(menu_item)\n\
</pre></div><p>For more information, see the following topics in the <em>MAXScript Help</em>:</p>\n\
<ul>\n\
<li>&quot;Menu Manager&quot;</li>\n\
<li>&quot;Menu and CUI Loading&quot;</li>\n\
<li>&quot;Interface: menuMan&quot;</li>\n\
<li>&quot;Macro Scripts&quot;</li>\n\
</ul>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";