var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>The pymxs Module</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>The pymxs Module</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"the-pymxs-module\"></a></div>\n\
<p>The pymxs module provides access to the MAXScript runtime, as well as some top-level functions and types. This topic covers those top-level members:</p>\n\
<ul>\n\
<li>The MAXScript runtime, <span class=\'code\'>pymxs.runtime</span></li>\n\
<li>Context expression functions, such as <span class=\'code\'>pymxs.animate</span></li>\n\
<li>The MAXScript undo system functions, <span class=\'code\'>run_undo()</span> and <span class=\'code\'>run_redo()</span></li>\n\
<li>The <span class=\'code\'>pymxs.print_()</span> function</li>\n\
<li>The <span class=\'code\'>pymxs.byref()</span> function, to handle those situations where a MAXScript function takes a by-reference parameter</li>\n\
<li>The <span class=\'code\'>pymxs.mxstoken()</span> function, to handle access to pymxs.runtime in multi-threaded situations</li>\n\
</ul>\n\
<!-- TOC -->\n\
<ul>\n\
<li><a href=\'pymxs_module.html#the-maxscript-runtime\' title=\'\'>The MAXScript Runtime</a></li>\n\
<li><a href=\'pymxs_module.html#context-expression-functions\' title=\'\'>Context Expression Functions</a></li>\n\
<li><a href=\'pymxs_module.html#maxscript-undo-and-redo-with-pymxsrun_undo-and-pymxsrun_redo\' title=\'\'>MAXScript Undo and Redo with <span class=\'code\'>pymxs.run_undo()</span> and <span class=\'code\'>pymxs.run_redo()</span></a></li>\n\
<li><a href=\'pymxs_module.html#printing-to-the-listener-with-pymxsprint_\' title=\'\'>Printing to the Listener with pymxs.print_()</a></li>\n\
<li><a href=\'pymxs_module.html#by-reference-parameter-handling\' title=\'\'>By-Reference Parameter Handling</a></li>\n\
<li><a href=\'pymxs_module.html#accessing-the-runtime-in-worker-threads-with-pymxsmxstoken\' title=\'\'>Accessing the Runtime In Worker Threads with <span class=\'code\'>pymxs.mxstoken()</span></a></li>\n\
</ul>\n\
<!-- /TOC -->\n\
<div class=\'section\'><a id=\"the-maxscript-runtime\"></a><h2 id=\"the-maxscript-runtime\">The MAXScript Runtime</h2></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime\n\
</pre></div><p>This member exposes the MAXScript runtime, and all the global symbols it contains, to Python scripts. The runtime is dynamic, so global objects created in MAXScript are subsequently available to <span class=\'code\'>pymxs.runtime</span>.</p>\n\
<p>The names of objects available in the <span class=\'code\'>pymxs.runtime</span> module correspond to MAXScript globals. So, for example, to create a teapot you can use the MAXScript <span class=\'code\'>Teapot()</span> constructor like this:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; import pymxs\n\
&gt;&gt;&gt; rt = pymxs.runtime\n\
&gt;&gt;&gt; t = rt.Teapot()\n\
&gt;&gt;&gt; rt.classOf(t)\n\
&lt;GeometryClass&lt;Teapot&gt;&gt;\n\
</pre></div><div class=\'section\'><a id=\"context-expression-functions\"></a><h2 id=\"context-expression-functions\">Context Expression Functions</h2></div>\n\
<p>Several MAXScript &quot;context expressions&quot; are exposed to pymxs via top-level functions. These members are:</p>\n\
<ul>\n\
<li><span class=\'code\'>pymxs.animate(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.attime(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.atlevel(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.quiet(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.redraw(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.undo(True|False)</span></li>\n\
</ul>\n\
<blockquote>\n\
<p><strong>Note:</strong> the following MAXScript context expressions are not available in pymxs: <span class=\'code\'>coordsys</span>, <span class=\'code\'>about</span>, <span class=\'code\'>printAllElements</span>, <span class=\'code\'>defaultAction</span>, <span class=\'code\'>MXSCallstackCaptureEnabled</span>, <span class=\'code\'>dontRepeatMessages</span>, and <span class=\'code\'>macroRecorderEmitterEnabled</span>. For <span class=\'code\'>coordsys</span>, you can achieve the same effect by changing the current coordinate system using <span class=\'code\'>pymxs.runtime.setRefCoordSys(mode_name)</span>.</p>\n\
</blockquote>\n\
<p>In MAXScript, a context expression is used in a scoped code block, for example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-mxs\">t = teapot()\n\
with animate on\n\
(\n\
  at time 0 t.position = [-100, 0, 0]\n\
  at time 100 t.position = [100, 0, 0]\n\
)\n\
</pre></div><p>Here, the <span class=\'code\'>with animate</span> context applies to the code block, while the <span class=\'code\'>at time</span> context applies to each statement. Note that the <span class=\'code\'>with</span> is optional in MAXScript, and you often do not see it used.</p>\n\
<p>In pymxs we would do something similar by scoping using the Python <span class=\'code\'>with</span> statement:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
t = rt.Teapot()\n\
\n\
with pymxs.animate(True):\n\
    with pymxs.attime(0):\n\
            t.pos = rt.Point3(-100, 0, 0)\n\
    with pymxs.attime(100):\n\
            t.pos = rt.Point3(100, 0, 0)\n\
</pre></div><div class=\'section\'><a id=\"maxscript-undo-and-redo-with-pymxs.run_undo-and-pymxs.run_redo\"></a><h2 id=\"maxscript-undo-and-redo-with-pymxs.run_undo-and-pymxs.run_redo\">MAXScript Undo and Redo with <span class=\'code\'>pymxs.run_undo()</span> and <span class=\'code\'>pymxs.run_redo()</span></h2></div>\n\
<ul>\n\
<li><span class=\'code\'>pymxs.undo(True|False)</span></li>\n\
<li><span class=\'code\'>pymxs.run_undo()</span></li>\n\
<li><span class=\'code\'>pymxs.run_redo()</span></li>\n\
</ul>\n\
<p>Most 3ds Max Commands (those invoked by <span class=\'code\'>max ...</span>) are not directly available in pymxs.runtime (see the pymxs Limitations and Workarounds topic). However, the two functions <span class=\'code\'>pymxs.run_undo()</span> and <span class=\'code\'>pymxs.run_redo()</span> are mapped to <span class=\'code\'>max undo</span> and <span class=\'code\'>max redo</span> respectively to provide direct access to these commands.</p>\n\
<p>As with MAXScript, operations run through pymxs are not recorded by the undo system. Undo must be specifically enabled. Undoable pymxs operations appear as items labeled <em>MAXScript</em> in the 3ds Max Undo menu.</p>\n\
<p>For example, to create a teapot, change its position, and then undo the change:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
# this will not be undoable:\n\
with pymxs.undo(False):\n\
    t = rt.Teapot()\n\
\n\
with pymxs.undo(True):\n\
    t.pos = rt.Point3(20,20,20)\n\
\n\
pymxs.run_undo() # undo the position assignment to [20,20,20]\n\
pymxs.run_redo() # redo the position assignement to [20,20,20]\n\
</pre></div><div class=\'section\'><a id=\"undo-redo-and-exceptions\"></a><h3 id=\"undo-redo-and-exceptions\">Undo / Redo and Exceptions</h3></div>\n\
<p>The way an undo block in pymxs handles exceptions is different than in MAXScript. When an exception is raised, code executed before the exception is undone. The exception is not raised beyond the scope of the undo block, as the undo operation is considered to be the exception handler. Here is an example to illustrate this behavior:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"># undo with exception\n\
\n\
import pymxs\n\
from pymxs import runtime as rt\n\
\n\
def make_box_and_raise():\n\
    with pymxs.undo(True, &#39;Making box&#39;):\n\
        rt.Box()\n\
        # this will undo the box, and the error will not show up\n\
        # in the listener, because it is handled:\n\
        raise RuntimeError(&#39;This is an error&#39;)\n\
        \n\
def make_sphere_and_raise():\n\
    rt.Sphere()\n\
    # this will show up in the lister as an unhandled exception\n\
    raise RuntimeError(&#39;This is an error&#39;)\n\
\n\
make_box_and_raise()\n\
make_sphere_and_raise()\n\
</pre></div><div class=\'section\'><a id=\"printing-to-the-listener-with-pymxs.print_\"></a><h2 id=\"printing-to-the-listener-with-pymxs.print_\">Printing to the Listener with pymxs.print_()</h2></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.print_(msg[, isErr[, [forceOnMainThread]])\n\
</pre></div><p>This member gives you more control over error formatting and threading behavior.</p>\n\
<ul>\n\
<li><span class=\'code\'>msg</span> - the string to print.</li>\n\
<li><span class=\'code\'>isErr</span> - an optional boolean that indicates whether the message should be formatted as an error (in red, by default). The default is False.</li>\n\
<li><span class=\'code\'>forceOnMainThread</span> - an optional boolean that indicates whether the print should happen on the main thread. In a threaded script, setting this to True ensures that output is sent to the Listener window even if called from a worker thread. If False, when called from a worker thread, output is sent to stdout/stderr (the <em>3dsmax_ouput.log</em> file or console). The default is True.</li>\n\
</ul>\n\
<p>This function exists because both MAXScript <span class=\'code\'>print()</span> and <span class=\'code\'>assert()</span> functions are disabled and not callable from <span class=\'code\'>pymxs.runtime</span>. There are a couple of ways to work around this limitation to send messages to the Listener window. The simplest is to use the standard Python <span class=\'code\'>print()</span> function and <span class=\'code\'>assert</span> statement. However, you cannot control formatting or threading behavior with these functions, so there may be situations where <span class=\'code\'>pymxs.print_()</span> is the better solution.</p>\n\
<div class=\'section\'><a id=\"by-reference-parameter-handling\"></a><h2 id=\"by-reference-parameter-handling\">By-Reference Parameter Handling</h2></div>\n\
<p>The <span class=\'code\'>pymxs.byref()</span> function allows you to use MAXScript functions that take &quot;by reference&quot; parameters. See the <a href=\'#!/url=./developer/using_pymxs/pymxs_differences/pymxs_by_ref_parameters.html\' title=\'\'>By-Reference Parameter Handling in pymxs</a> topic for more information.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> As of <strong>3ds Max 2021</strong>, the <span class=\'code\'>pymxs.mxsreference()</span> function is marked as deprecated, and will produce a warning in the Listener window when called the first time. Use <span class=\'code\'>pymxs.byref()</span> instead.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"accessing-the-runtime-in-worker-threads-with-pymxs.mxstoken\"></a><h2 id=\"accessing-the-runtime-in-worker-threads-with-pymxs.mxstoken\">Accessing the Runtime In Worker Threads with <span class=\'code\'>pymxs.mxstoken()</span></h2></div>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.mxstoken()\n\
</pre></div><p>You cannot call <em>pymxs.runtime</em> commands directly on a worker thread, as this will cause a runtime exception. However, <em>pymxs</em> has a <span class=\'code\'>mxstoken()</span> function to support multi-threading by providing a synchronization mechanism between worker threads.</p>\n\
<p>For an example of this approach, see the example <strong>\\<maxdirectory\\>\\scripts\\Python\\demoMXSToken.py</maxdirectory\\></strong>.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";