var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Global and Persistent Variables in pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Global and Persistent Variables in pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"global-and-persistent-variables-in-pymxs\"></a></div>\n\
<p>Global variables in MAXScript are &quot;top level&quot; variables that have global scope. They can be explicitly declared (using the <span class=\'code\'>global</span> keyword), or implicitly declared outside of any scope. See the &quot;Scope of Variables&quot; topic in the MAXScript Help.</p>\n\
<p>Persistent variables are global variables that are attached to a scene and are retained on scene close and open, and between 3ds Max sessions. They are created either by using the <span class=\'code\'>persistent</span> keyword when declaring the variable, or by using <span class=\'code\'>peristents.make()</span> on the name of an existing global variable. See the &quot;Persistent Global Variables&quot; topic in the MAXScript Help.</p>\n\
<p>In this example, we create a few global variables, and then make them persistent.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"># Globals and Persistents\n\
\n\
from pymxs import runtime as rt\n\
\n\
# As of 3ds Max 2021, we can implictly create variables from Python:\n\
rt.a = &#39;hello&#39;\n\
\n\
# Pre-2021, you need to use pymxs.runtime.execute\n\
# this is one way to explicitly declare a global variable:\n\
rt.execute(&#39;global b = &quot;hello again&quot;&#39;)\n\
\n\
# Any variable declared outside of a scope is implictly considered global,\n\
# so this also works:\n\
rt.execute(&#39;c = &quot;goodbye&quot;&#39;)\n\
\n\
# To view all globals currently declared:\n\
globals = rt.globalVars.gather()\n\
print (globals)\n\
\n\
# getting and setting values requires a MAXScript Name \n\
val = rt.globalVars.get(rt.name(&#39;a&#39;))\n\
print (&#39;a={}&#39;.format(val))\n\
\n\
# Persistent variables can be created on declaration, or made persistent later on\n\
# Re-declare a:\n\
rt.execute(&#39;persistent global a = &quot;hello&quot;&#39;)\n\
\n\
# persistents.make() takes a name value of an existing global variable, and\n\
# is preferable because it does not require pymxs.runtime.execute():\n\
rt.persistents.make(rt.name(&#39;b&#39;)) \n\
\n\
# We can get all the global persistent variables with persistents.gather\n\
val = rt.persistents.gather()\n\
print (&#39;gather={}&#39;.format(val))\n\
</pre></div><p>The output is:</p>\n\
<div class=\'codeBlock\'><pre>#(#NodeColorPicker, #getModContextBBoxMax, #Summed_area, #MAXCurveCtl, #setSplitMesh, #arnold_spherical_camera, #joinSurfaces, #StereoCameraRig, #log, #ai_fraction, #PxStepSimulation, #renderers, #PFBoxMeshWrapper, #Light__Area, #fetchMaxFile, #getSplitMesh, #Color_RGB, #Brightness_and_Contrast, #MSPluginClass, #g_ep, ...)\n\
a=hello\n\
gather=#(#b, #CAT_UINum, #a)\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";