var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Registering Python Callbacks</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Registering Python Callbacks</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"registering-python-callbacks\"></a></div>\n\
<p>Some MAXScript callback registration functions can take a Python function as the callback argument. These include <span class=\'code\'>registerTimeCallback()</span>, <span class=\'code\'>registerRedrawViewsCallback()</span>, <span class=\'code\'>nodeEventCallback()</span>, and <span class=\'code\'>callbacks.addScript()</span> (as of <strong>3ds Max 2021</strong>). The exception is <span class=\'code\'>DialogMonitorOPS.RegisterNofication()</span>, which takes a string argument, and thus cannot take a Python function.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> The <a href=\'#!/url=./developer/notification_codes.html\' title=\'\'>Notification Codes</a> topic lists available event codes for both MaxPlus and pymxs, and is useful for migrating scripts from MaxPlus to pymxs.</p>\n\
</blockquote>\n\
<p>For example, you can use the general callback system to create a callback that fires when nodes are created:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
# remove any previous callbacks:\n\
rt.callbacks.removeScripts(id=rt.name(&#39;MyCallbacks&#39;))\n\
\n\
# function to print the notification param, which in the case of NodeCreated, is the node\n\
def myCallback():\n\
    print (&#39;Callback fired!&#39;)\n\
    notification = rt.callbacks.notificationParam()\n\
    print (notification)\n\
\n\
rt.callbacks.addScript(rt.Name(&#39;nodeCreated&#39;), myCallback, id=rt.Name(&#39;MyCallbacks&#39;))\n\
# Will print something like this:\n\
# Callback fired!\n\
# $Sphere:Sphere005 @ [0.000000,0.000000,0.000000]\n\
\n\
print(rt.callbacks.show(id=rt.Name(&#39;MyCallbacks&#39;), asArray=True))\n\
\n\
# Prints something like:\n\
# #(#(#nodeCreated, #myCallbacks, false, false, &quot;&lt;function myCallback at 0x0000021020609828&gt;()&quot;))\n\
\n\
\n\
</pre></div><p>We need to use the <span class=\'code\'>asArray=True</span> parameter for <span class=\'code\'>callbacks.show()</span>, because normally this function tries to output to <span class=\'code\'>stdout</span> (the listener), which is disabled in Python for this call. You could also use the <span class=\'code\'>to:stringStream</span> parameter to capture the output.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> Even though the <span class=\'code\'>id</span> parameter is optional for <span class=\'code\'>addScript()</span>, it is a best practice to always provide one, to make it easier to find and remove callbacks later. In this example we can use it to &quot;clean up&quot; before running the script.</p>\n\
</blockquote>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";