var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Working with the Renderer</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Working with the Renderer</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"working-with-the-renderer\"></a></div>\n\
<p>This topic discusses ways to perform rendering tasks using pymxs.</p>\n\
<p>{toc}</p>\n\
<div class=\'section\'><a id=\"executing-your-render\"></a><h2 id=\"executing-your-render\">Executing your render</h2></div>\n\
<p>The <span class=\'code\'>render()</span> function invokes the 3ds Max renderer, and takes many optional parameters to control the rendering independent of the current <strong>Render Settings Dialog</strong> settings.</p>\n\
<p>By default this function returns an in-memory render as a bitmap value.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"># this will display the render in a virtual file buffer window\n\
bmp = pymxs.runtime.render(pos=pymxs.runtime.name(&#39;vfb_upper_left&#39;))\n\
# the bitmap can also be displayed later:\n\
pymxs.runtime.display(bmp, caption=&#39;My Bitmap&#39;)\n\
</pre></div><div class=\'section\'><a id=\"rendering-to-an-output-file\"></a><h2 id=\"rendering-to-an-output-file\">Rendering to an output file</h2></div>\n\
<p>The <span class=\'code\'>render()</span> function can save the render to a file if the <span class=\'code\'>outputFile:</span> parameter is specified.</p>\n\
<p>For example, this function (from the <span class=\'code\'>render.py</span> pymxs sample) saves the rendered output as a single-frame jpeg file to the system default <span class=\'code\'>#renderoutput</span> directory, removing the current render if it exists:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">def render():\n\
    &#39;&#39;&#39;Render in the renderoutput directory.&#39;&#39;&#39;\n\
    output_path = os.path.join(pymxs.runtime.getDir(pymxs.runtime.Name(&quot;renderoutput&quot;)), &#39;foo.jpg&#39;)\n\
    if os.path.exists(output_path):\n\
        os.remove(output_path)\n\
    pymxs.runtime.render(outputFile=output_path)\n\
</pre></div><div class=\'section\'><a id=\"using-pre--and-post-render-scripts\"></a><h2 id=\"using-pre--and-post-render-scripts\">Using pre- and post-render scripts</h2></div>\n\
<p>There are two mechanisms to register pre- and post-render scripts: - set the globals <span class=\'code\'>preRendScript</span> (string) and <span class=\'code\'>usePreRendScript</span> (boolean), or <span class=\'code\'>postRendScript</span> (string) and <span class=\'code\'>usePostRendScript</span> (boolean), which correspond to the settings under <strong>Render Setup Dialog &gt; Common Parameters &gt; Scripts</strong> in the user interface. This mechanism only works when the user presses the render button in the UI, these scripts are not called when the <span class=\'code\'>pymxs.runtime.render()</span> function is called. - register a #preRender or #postRender callback. This script will be called when <span class=\'code\'>pymxs.runtime.render()</span> is called.</p>\n\
<p>An example of the latter mechanism:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">mxs = pymxs.runtime\n\
\n\
def myPreRenderCallback():\n\
    print(&#39;PreRender Callback Called&#39;)\n\
    \n\
mxs.callbacks.removeScripts(id=mxs.Name(&#39;MyCallbacks&#39;))    \n\
mxs.callbacks.addScript(mxs.name(&#39;preRender&#39;), myPreRenderCallback, id=mxs.name(&#39;MyCallbacks&#39;))\n\
# this will run the registered script first:\n\
mxs.render()\n\
</pre></div><div class=\'section\'><a id=\"rendering-settings-and-dialogs\"></a><h2 id=\"rendering-settings-and-dialogs\">Rendering Settings and Dialogs</h2></div>\n\
<p>You can change rendering settings using the <span class=\'code\'>pymxs.runtime.renderSceneDialog</span> struct. Note that changes to render settings should be made with the dialog closed (which you can check with the <span class=\'code\'>renderSceneDialog.isOpen()</span> function). See the &quot;Render Scene Dialog&quot; group of topics in the MAXScript Help for information about properties and functions in this struct.</p>\n\
<p>The renderer assigned to each type of render target is held in the renderers struct. For example, by default, renderers.production is the scanline renderer:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; pymxs.runtime.renderers.production\n\
&lt;Default_Scanline_Renderer&lt;Default_Scanline_Renderer:Default_Scanline_Renderer&gt;&gt;\n\
</pre></div><p>The list of all available renderer classes is held in the <span class=\'code\'>rendererClass.classes</span> list, and can be assigned to a render target:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; pymxs.runtime.renderers.current = pymxs.runtime.rendererClass.classes[5]()\n\
<blockquote>\n\
</blockquote></pre></div><p><strong>Note</strong>: Because this list is accessed via Python, it is 0-based. Also note that you create an instance of the renderer class using the constructor notation.</p>\n\
\n\
<div class=\'section\'><a id=\"working-with-the-viewport\"></a><h2 id=\"working-with-the-viewport\">Working with the viewport</h2></div>\n\
<p>The <span class=\'code\'>viewport</span> struct exposes properties and methods for working with viewports.</p>\n\
<div class=\'section\'><a id=\"iterating-through-the-list-of-viewports\"></a><h3 id=\"iterating-through-the-list-of-viewports\">Iterating through the list of viewports</h3></div>\n\
<p>You can use the <span class=\'code\'>viewport.numviews</span> property to iterate through the available views. For example, to list the viewports and their current layout types:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">mxs = pymxs.runtime\n\
for v in range(1,mxs.viewport.numviews+1):\n\
    viewtype = mxs.viewport.getType(index=v)\n\
    print(f&#39;index:{v} view type:{viewtype}&#39;)\n\
</pre></div><p>Note: We use a range of 1-numviews+1 because MAXScript is 1-based.</p>\n\
<div class=\'section\'><a id=\"maximizing-and-zooming-a-viewport\"></a><h3 id=\"maximizing-and-zooming-a-viewport\">Maximizing and zooming a viewport</h3></div>\n\
<p>The MAXScript command to maximize or minimize the current viewport is a &quot;Max Command&quot;, in the form of:</p>\n\
<div class=\'codeBlock\'><pre>max tool maximize\n\
</pre></div><p>As noted in <a href=\'#!/url=./developer/using_pymxs/pymxs_differences/pymxs_max_commands.html\' title=\'\'>Calling 3ds Max Commands</a>, Max commands are not directly callable in pymxs, so to invoke this command, use:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.execute(&#39;max tool maximize&#39;)\n\
</pre></div><p>Setting the zoom level of the current viewport is exposed through the <span class=\'code\'>viewport.zoom()</span> function, which takes a float value setting the zoom factor, where 2.0 zooms &quot;out&quot; two times, while 0.5 zooms &quot;in&quot; two times. If the viewport is in perspective mode, you can also use <span class=\'code\'>viewport.zoomPerspective()</span>.</p>\n\
<p>To &quot;zoom extents&quot;, once again a Max command is used:</p>\n\
<div class=\'codeBlock\'><pre>max tool zoomextents\n\
max tool zoomextents all\n\
</pre></div><div class=\'section\'><a id=\"saving-the-active-viewport-graphics-window-as-a-bitmap\"></a><h3 id=\"saving-the-active-viewport-graphics-window-as-a-bitmap\">Saving the active viewport graphics window as a bitmap</h3></div>\n\
<p>The <span class=\'code\'>viewport.getViewportDib()</span> function returns a snapshot of a viewport&#39;s window image as bitmap. If no additional parameters are provided, it returns a bitmap for the currently active viewport. It takes these optional parameters:</p>\n\
<ul>\n\
<li><span class=\'code\'>index</span> - specifies the index of the viewport to capture; the default is the currently active viewport</li>\n\
<li><span class=\'code\'>viewPanelIndex</span> - specifies the index of the view panel to capture; the default is the currently active view panel</li>\n\
<li><span class=\'code\'>captureAlpha</span> (boolean) - specifies whether to capture the Alpha channel as well (ARGB); the default is False</li>\n\
<li><span class=\'code\'>gammaCorrect</span> (boolean) - specifies whether gamma correction is performed; the default is True</li>\n\
</ul>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">my_bitmap = pymxs.runtime.viewport.getViewportDib(captureAlpha=True)\n\
pymxs.runtime.display(my_bitmap)\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";