var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Working with the Scene Graph</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Working with the Scene Graph</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"working-with-the-scene-graph\"></a></div>\n\
<p>The scene graph is a tree of scene nodes. Each node is a hub of information for objects in the scene, and is responsible for:</p>\n\
<ul>\n\
<li>applying modifiers to an object</li>\n\
<li>storing information about the geometry</li>\n\
<li>storing information about shading materials</li>\n\
<li>storing position, rotation, and scale information</li>\n\
<li>storing node tree hierarchical information: the parent and child of the node</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"traversing-the-scene-graph\"></a><h2 id=\"traversing-the-scene-graph\">Traversing the Scene Graph</h2></div>\n\
<p>The root node of a scene can be accessed by the MAXScript <span class=\'code\'>rootNode</span> global. See, for example, the <em>scene_graph.py</em> sample script:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&#39;&#39;&#39;\n\
    Creates a simple text representation of the scene graph\n\
&#39;&#39;&#39;\n\
from pymxs import runtime as rt \n\
\n\
def output_node(node, indent=&#39;&#39;):\n\
    &quot;&quot;&quot;Print the scene graph as text to stdout.&quot;&quot;&quot;\n\
    print(indent, node.Name)\n\
    for child in node.Children:\n\
        output_node(child, indent + &#39;--&#39;)\n\
\n\
output_node(rt.rootnode)\n\
</pre></div><div class=\'section\'><a id=\"getting-scene-objects\"></a><h2 id=\"getting-scene-objects\">Getting Scene Objects</h2></div>\n\
<p>If you do not need the scene hierarchy information (that is, parent/child relationships), you can get a flat list of all objects in the scene with the <span class=\'code\'>objects</span> global variable. For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">objs = pymxs.runtime.objects\n\
for o in objs:\n\
    print(o.name)\n\
</pre></div><p>You can also obtain individual objects by name:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">s = pymxs.runtime.getNodeByName(&#39;Sphere002&#39;)\n\
</pre></div><p>Finally, you can also access objects by PathName values, though this is not directly exposed to pymxs and requires a work-around. See <a href=\'#!/url=./developer/using_pymxs/pymxs_differences/pymxs_pathname_values.html\' title=\'\'>Accessing MAXScript PathName Values</a> for more information.</p>\n\
<div class=\'section\'><a id=\"getting-the-selection-list\"></a><h2 id=\"getting-the-selection-list\">Getting the Selection List</h2></div>\n\
<p>There are times when it is useful to get the list of selected objects in a scene.</p>\n\
<p>There are several reserved system variables of type <span class=\'code\'>ObjectSet</span> that contain various scene object categories. For example, <span class=\'code\'>objects</span> contains all the objects in the scene, <span class=\'code\'>lights</span> all the lights, and selection contains the current <span class=\'code\'>selection</span>.</p>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; pymxs.runtime.selection\n\
&lt;ObjectSet&lt;$selection&gt;&gt;\n\
&gt;&gt;&gt; pymxs.runtime.selection[0]\n\
&lt;Sphere&lt;$Sphere:Sphere004 @ [90.515228,15.940669,0.000000]&gt;&gt;\n\
&gt;&gt;&gt;\n\
</pre></div><p>Named selection sets are contained in the <span class=\'code\'>selectionSets</span> array, and are accessed by index.</p>\n\
<p>for example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; pymxs.runtime.selectionSets[0]\n\
&lt;SelectionSet&lt;SelectionSet:my_selection&gt;&gt;\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";