var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>MAXScript Values in pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>MAXScript Values in pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"maxscript-values-in-pymxs\"></a></div>\n\
<p>This topic covers how Python and MAXScript values and types interoperate with pymxs.</p>\n\
<div class=\'section\'><a id=\"maxscript-type-marshalling\"></a><h2 id=\"maxscript-type-marshalling\">MAXScript Type Marshalling</h2></div>\n\
<p>For simple types such as integers, floats and strings, <span class=\'code\'>pymxs</span> has a direct reference to the data, and these values can be obtained and set directly.</p>\n\
<p>For complex types such as Array, pymxs wraps these objects with a <span class=\'code\'>MXSWrapper</span> object, <span class=\'code\'>pymxs.MXSWrapperBase</span>. For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
# Create and visit a MaxScript Array\n\
gma = rt.array(*(1,2,3))\n\
print(gma)    #output: #(1,2,3)\n\
print(len(gma))    #output: 3\n\
gma[1] = 999\n\
print(gma[1])    #output: 999\n\
l = list(gma) #[1,999,3]\n\
l[2] = 888\n\
print(l)    #output: [1, 888, 3]\n\
print(gma)   # output: #(1, 999, 3)\n\
</pre></div><p>For a demonstration of how to create, manipulate and cast complex MAXScript types in Python, see the <em>demoPyMXSTypeInterop.py</em> example.</p>\n\
<div class=\'section\'><a id=\"dictionaries\"></a><h2 id=\"dictionaries\">Dictionaries</h2></div>\n\
<p>MAXScript introduced the Dictionary type in <strong>3ds Max 2017.1</strong>, and by default uses name literals for key values.</p>\n\
<p>Consider this MAXScript dictionary:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-mxs\">my_dict = Dictionary #(#one, 1) #(#two, 2)\n\
</pre></div><p>In pymxs, this value is wrapped, and can be used like a MAXScript dictionary, for example getting the keys property:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; my_py_dict = pymxs.runtime.my_dict\n\
&lt;Dictionary&lt;Dictionary #name one:1 two:2 &gt;&gt;\n\
&gt;&gt;&gt; my_py_dict.keys\n\
&lt;Array&lt;#(#one, #two)&gt;&gt;\n\
</pre></div><p>In some cases, it may be more convenient to convert the MAXScript Dictionary to a native Python Dict:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">my_py_dict = {str(key):myd[key] for key in pymxs.runtime.my_dict.keys}\n\
# yields: {&#39;two&#39;: 2, &#39;one&#39;: 1}\n\
</pre></div><div class=\'section\'><a id=\"maxscript-collections-in-pymxs\"></a><h2 id=\"maxscript-collections-in-pymxs\">MAXScript Collections in pymxs</h2></div>\n\
<p>MAXScript arrays and dictionaries can be created and manipulated in pymxs. In <strong>3ds Max 2021</strong> and later you can manipulate a MAXScript Array directly.</p>\n\
<p>This example illustrates how to create and manipulate MAXScript arrays and dictionaries in pymxs:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"># MAXScript Collections in pymxs\n\
\n\
from pymxs import runtime as rt\n\
import pymxs\n\
\n\
py_array = rt.Array(1,2,3)\n\
# 3ds Max 2020 and earlier requires rt.append\n\
rt.append(py_array, 4)\n\
\n\
try:\n\
    # In 3ds Max 2021, we can do this directly:\n\
    # adding an arbitrary element will created &quot;undefined&quot; intermediary elements:\n\
    py_array[5] = 5\n\
except:\n\
    print(&#39;Direct access is available in 3ds Max 2021+&#39;)\n\
\n\
py_dict = rt.Dictionary((&quot;one&quot;,1), (&quot;two&quot;,2))\n\
py_dict[&quot;three&quot;] = 3\n\
# you could also use rt.SetDictValue(py_dict, &#39;three&#39;, 3)\n\
\n\
print(&quot;py_array: &quot;, py_array)\n\
print(&quot;py_dict: &quot;, py_dict)\n\
</pre></div><p>Results:</p>\n\
<div class=\'codeBlock\'><pre>py_array:  #(1, 2, 3, 4, undefined, 5)\n\
py_dict:  Dictionary #string (DataPair &quot;three&quot; 3) (DataPair &quot;one&quot; 1) (DataPair &quot;two&quot; 2) \n\
</pre></div><div class=\'section\'><a id=\"using-pymxs-objects-as-python-dictionary-keys\"></a><h2 id=\"using-pymxs-objects-as-python-dictionary-keys\">Using pymxs objects as Python Dictionary keys</h2></div>\n\
<p>As of 3ds Max 2019 Update 1, you can use some pymxs objects (such as scene nodes) as key values in native Python dictionaries (though not in MAXScript Dictionaries). For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
t1 = rt.teapot()\n\
t2 = rt.teapot(pos=rt.point3(20,20,0))\n\
t2.parent = t1\n\
\n\
node_dict = {t1:&quot;Teapot 1&quot;, t2:&quot;Teapot2&quot;}\n\
\n\
print(node_dict.keys())\n\
print(node_dict.values())\n\
</pre></div><p>To determine whether a MAXScript object can be used as a Python dictionary key, you can check whether it implements the python <span class=\'code\'>__hash__()</span> function.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";